@@ -5,6 +5,7 @@ from __future__ import absolute_import, print_function, division, unicode_litera
 import inspect
 import re
 import six
+from io import BufferedReader, BytesIO
 
 import pytest
 import requests
@@ -334,6 +335,20 @@ def test_match_querystring_auto_activates():
     assert_reset()
 
 
+def test_match_querystring_missing_key():
+    @responses.activate
+    def run():
+        responses.add(responses.GET, "http://example.com?foo=1&bar=2", body=b"test")
+        with pytest.raises(ConnectionError):
+            requests.get("http://example.com/?foo=1&baz=2")
+
+        with pytest.raises(ConnectionError):
+            requests.get("http://example.com/?bar=2&fez=1")
+
+    run()
+    assert_reset()
+
+
 def test_accept_string_body():
     @responses.activate
     def run():
@@ -909,6 +924,36 @@ def test_handles_unicode_url():
     assert_reset()
 
 
+def test_handles_unicode_body():
+    url = "http://example.com/test"
+
+    @responses.activate
+    def run():
+        responses.add(responses.GET, url, body="михољско лето")
+
+        resp = requests.get(url)
+
+        assert_response(resp, "михољско лето", content_type="text/plain; charset=utf-8")
+
+    run()
+    assert_reset()
+
+
+def test_handles_buffered_reader_body():
+    url = "http://example.com/test"
+
+    @responses.activate
+    def run():
+        responses.add(responses.GET, url, body=BufferedReader(BytesIO(b"test")))
+
+        resp = requests.get(url)
+
+        assert_response(resp, "test")
+
+    run()
+    assert_reset()
+
+
 def test_headers():
     @responses.activate
     def run():
@@ -971,6 +1016,21 @@ def test_multiple_urls():
     assert_reset()
 
 
+def test_multiple_methods():
+    @responses.activate
+    def run():
+        responses.add(responses.GET, "http://example.com/one", body="gotcha")
+        responses.add(responses.POST, "http://example.com/one", body="posted")
+
+        resp = requests.get("http://example.com/one")
+        assert_response(resp, "gotcha")
+        resp = requests.post("http://example.com/one")
+        assert_response(resp, "posted")
+
+    run()
+    assert_reset()
+
+
 def test_passthru(httpserver):
     httpserver.serve_content("OK", headers={"Content-Type": "text/plain"})
 
